
# Goal: Calculate solar amount cues for all combinations of n1 & n2 days.
# Requires values for n1 & n2 & input files cd & sun.
# cd holds census, date.mean & ndays & is ordered chronologically.
#     census values are increasing integers (any missing integers will be accomodated) 
#          (first row predates first census by 1 census interval),
#     date.mean is not used, &
#     ndays is the number of days since 1980-01-01.
# sun holds V1 to V361 & is ordered chronologically.
#     VX is cumulative insolation (MJ/m2) over X days before V361 (1<=X<=360),
#     V361 is number of days since 1980-01-01.
# The output file holds census, ndays, N1, N2, Mean.100 & Max.100.
#     census values are increasing integers 
#     ndays is the mean date of the census as number of days since 1980-01-01.
#     N1 is the number of days between flower fall & the end of the proximate cue
#     N2 is the number of days over which the proximate cue accumulates
#     Mean.100 is the average over the days in the census interval of insolation N1+1 
#         to N1+N2 days earlier (MJ/m2 x 100). 
#     Max.100 is the maximum of Mean.100 over days in the census interval (MJ/m2 x 100).

setwd("C:/Users/wrightj/Dropbox/Clouds/RMSE/Dryad")
outfile = "Sun_Amount_Observed.txt"

# Define n1 & n2. 
N1=seq(1,210,1)
N2=seq(1,125,1)

# Input census mean dates (cd) and add begin phenological years (BPY). 
# BPY holds the census that Begins a Phenological Year preceding the current census.
# Phenological years begin in late Oct, when solar inputs are at their annual low.
# BeginYr holds the final censuses from each October.
cd = read.table("CensusMeanDates.txt",header=T,sep="\t")
cd = cd[,c(1,3)] # remove unused date variable
cd$Di = NA # define initial day of census 
for (i in 2:nrow(cd)) {cd$Di[i] = cd$ndays[i-1]} # define initial day of census
BeginYr = c(0,43,96,147,199,251,303,355,407,460,512,564,616,668,721,773,825,
            877,929,981,1034,1086,1138,1190,1242,1295,1347,1399,1451,1503,1555)
cd$BPY = BeginYr[length(BeginYr)]
for (i in 1:nrow(cd)) {
  for (j in 2:length(BeginYr)) {
    cd$BPY[i] = ifelse(cd$census[i]>BeginYr[j-1] & cd$census[i]<=BeginYr[j], BeginYr[j-1], cd$BPY[i])
  }
}
cd = cd[cd$census>=6,] # flower records begin with census 7
cd = as.matrix(cd)

# Input the environmental file (sun).
sun = read.table("LookUp_Irradiance.txt",header=T,sep="\t")
sun = sun[,c(1:(max(N1)+max(N2)),361)] # remove unused columns
sun = sun[sun[,ncol(sun)]>=cd[1,2] & sun[,ncol(sun)]<=cd[nrow(cd),2],] # remove unused rows
sun = as.matrix(sun)

# Create look up table for all pairwise values of N1 & N2.
result = matrix(nrow=(max(N1)-min(N1)+1)*(max(N2)-min(N2)+1)*(nrow(cd)-1),ncol=7) # Note nrow(cd)-1 
counter = 0
ndays = sun[,"V362"]
Di = cd[,"Di"]
Df = cd[,"ndays"]
for (n2 in min(N2):max(N2)) {
  print(n2)
  for (n1 in min(N1):max(N1)) {
    for (C in 2:nrow(cd)) {
      counter = counter+1
      condition = ndays>=Di[C] & ndays<Df[C]
      v = (sun[condition,(n1+n2)] - sun[condition,n1])/n2
      avg = mean(v)
      v = max(v) 
      result[counter,] = c(cd[C,"census"],cd[C,"ndays"],cd[C,"BPY"],n1,n2,avg,v)      
    }
  }
}
result=as.data.frame(result)
colnames(result) = c("census","ndays","BPY","n1","n2","Mean.100","Max.100")
result$census = as.integer(result$census)
result$ndays = as.integer(result$ndays)
result$BPY = as.integer(result$BPY)
result$n1 = as.integer(result$n1)
result$n2 = as.integer(result$n2)
result$Mean.100 = as.integer(100*result$Mean.100)
result$Max.100 = as.integer(100*result$Max.100)
write.table(result,outfile,row.names=F,sep="\t")
