
# Goal: Calculate root mean square error (RMSE) for all combinations of n1 
#       & n2 days & threshold rainfall amounts. User specifies 
#       1. whether threshold refers to amounts or slopes, 
#       2. whether average or maximum value of threshold is used, & 
#       3. the range of observed values to compare with the threshold.
# Strategy: BeginYr holds censuses that begin phenological years.
#           Predicted first flowering censuses are the first censuses that 
#           rainfall exceeds the threshold after BeginYr for each year.
# Requires CensusMeanDates.txt, Median_First_Census.txt & 
#          Rain_Amount_Observed.txt or Rain_Slope_Observed.txt.
# CensusMeanDates holds census, date.mean & ndays & is ordered chronologically.
#    census values are increasing integers (missing integers are accomodated) 
#         (first row predates first census by 1 census interval),
#    date.mean (format is "YYYY-MM-DD HH:MM:SS"),
#    ndays is the number of days since 1980-01-01.
# Median_First_Census holds sp, yr & census.
#    sp is a four-letter mnemonic that identifies the 19 study species
#    yr is the calendar year flowering year began
#    census is the median first census over traps for the sp & yr.
# Rain_Amount_Cues holds census, ndays, N1, N2, mean.rain & max.rain.
#    census values are increasing integers 
#    ndays is the mean date of the census as number of days since 1980-01-01.
#    N1 is number of days between flower fall & the end of the proximate cue
#    N2 is number of days over which the proximate cue accumulates
#    Mean.100 is average daily rain from N1+1 to N1+N2 days earlier (mm x 100).
#    Max.100 is the maximum value of Mean.100 (mm x 100).
# The output file holds sp, N1, N2, RMSE, NYrs & threshold.
#    sp is a four-letter mnemonic that identifies the 19 study species
#    N1 is number of days between flower fall & the end of the proximate cue
#    N2 is number of days over which the proximate cue accumulates
#    RMSE is root mean square error
#    NYrs is number of years with observed & predicted first flowering censuses.
#    threshold is the value of threshold exceeded by observed conditions.

# 1. Specify whether the threshold concerns amounts or slopes.
# amount.slope = "slope"
amount.slope = "amount"
# 2. Specify average or maximum value of threshold.
# avg.max = "maximum"
avg.max = "average"
# 3. Specify range of observed values that cues flowering by exceeding threshold.
LOW = 5
HIGH = 11.1
STEP = 0.1

setwd("C:/Users/wrightj/Dropbox/Clouds/RMSE/Dryad")
infile1 = "CensusMeanDates.txt"
infile2 = "Median_First_Flowering_Census.txt"
if (amount.slope = "amount") infile3 = "Rain_Amount_Observed.txt"
if (amount.slope = "slope") infile3 = "Rain_Slope_Observed.txt"
if (amount.slope = "amount") outfile = "RMSE_Rain_Amount.txt"
if (amount.slope = "slope") outfile = "RMSE_Rain_Slope.txt"

# Select the first census of the year to begin phenological years.
cd = read.table(infile1,header=T,sep="\t")
cd$time = strptime(cd$date.mean,format="%Y-%m-%d %H:%M:%S")
cd$mark = T
for (i in 2:dim(cd)[1]) {cd$mark[i] = ifelse(cd$time$year[i-1]==cd$time$year[i], F, T)}
BeginYr = cd$census[cd$mark==T]

# Add BPY to observed median first flowering censuses
a = read.table(infile2,header=T,sep="\t")
a$sp = as.character(a$sp)
a = a[order(a$sp,a$yr),]
nyr = length(BeginYr)
for (i in 1:nrow(a)) {
  for (j in 1:nyr) {if (a$census[i]>BeginYr[j]) a$BPY[i] = BeginYr[j]}
}
for (i in 2:nrow(a)) {
  con = a$BPY[i]==a$BPY[i-1]
  if (con) for (j in 2:(nyr-1)) {if (a$BPY[i]==BeginYr[j]) holdj=j}
  if (con & (a$BPY[i+1]-a$BPY[i])<75 & (a$sp[i+1]==a$sp[i])) a$BPY[i-1] = BeginYr[holdj-1]  
  if (con & ((a$BPY[i+1]-a$BPY[i])>75 | (a$sp[i+1]!=a$sp[i]))) a$BPY[i] = BeginYr[holdj+1]
}
a = a[,c("sp","census","BPY")]

# Calculate RMSE.
b = read.table(infile3,header=T,sep="\t")
if (avg.max="average") b = b[,c("census","BPY","n1","n2","Mean.100")]
if (avg.max="maximum") b = b[,c("census","BPY","n1","n2","Max.100")]
names(b) = c("census","BPY","n1","n2","observed")
BPY = b[,"census"] %in% BeginYr 
result = data.frame()
for (threshold in seq(LOW,HIGH,STEP)) { 
  t = b[,"observed"]>threshold 
  p = rep(F,nrow(b))
  if (b[1,"census"]>BeginYr[1]) begin = 1 # Initialize begin.
  for (i in 1:nrow(b)) {
    if (BPY[i]) begin = 1
    if ((begin + t[i])==2) {p[i]=T; begin=0}
  }
  predict = b[p,c("census","n1","n2","BPY")]
  d = merge(a,predict,by="BPY",all.x=T)
  e = aggregate((d$census.x-d$census.y)^2,by=list(d$sp,d$n1,d$n2),sum)
  f = as.data.frame(table(d$sp,d$n1,d$n2))
  f = f[f$Freq!=0,] # Object e lacks parameter sets that never exceed threhsold.
  g = cbind(e,f$Freq)
  names(g) = c("sp","n1","n2","RMSE","NYrs")
  g$RMSE = sqrt(g$RMSE/g$NYrs)
  g$threshold = threshold
  result = rbind(result,g)
}
result$hypothesis = paste(avg.max,amount.slope,sep="_")
write.table(result,outfile,row.names=F,sep="\t")
